@@ -2,20 +2,37 @@ import logging
 import sys
 import traceback
 import uuid
+from enum import Enum
 from inspect import signature
 from io import StringIO
 from typing import List
 from uuid import uuid4
 
+import yaml
+
 from kodexa.connectors import FolderConnector
 from kodexa.model import Document
-from kodexa_cloud import ContentObject
 
 
 def new_id():
     return str(uuid.uuid4()).replace("-", "")
 
 
+class ContentType(Enum):
+    DOCUMENT = 'DOCUMENT'
+    NATIVE = 'NATIVE'
+
+
+class ContentObject:
+
+    def __init__(self, name, id=new_id(), contentType=ContentType.DOCUMENT, tags=[], metadata={}):
+        self.id = id
+        self.name = name
+        self.contentType = contentType
+        self.tags = tags
+        self.metadata = metadata
+
+
 class InMemoryContentProvider:
     """
     A content provider is used to support getting content (documents or native) to
@@ -210,6 +227,26 @@ class Pipeline:
 
         return self
 
+    def to_yaml(self):
+        """
+        Will return the YAML representation of any actions that support conversion to YAML
+
+        The YAML representation for KodexaAction's can be used for metadata only pipelines in the Kodexa Platform
+
+        :return: YAML representation
+        """
+
+        configuration_steps = []
+
+        for step in self.steps:
+
+            try:
+                configuration_steps.append(step.to_configuration())
+            except:
+                pass
+
+        return yaml.dump(configuration_steps)
+
     def run(self):
         """
         Run the current pipeline, note that you must have a sink in place to allow the pipeline to run
